#!/bin/bash
core_user="core_user"

php_app_destination="/var/www/html/cloudbox"
vendor_files="vendor/jfrog/insight-core"

mkdir -p $JFI_HOME
mkdir -p $JFI_HOME_CORE

#Given a file, this function runs the jq command and saves it back
replace_value() {
	file_name=$1
	jq_command=$2
	cat $file_name|jq $jq_command > tmpfile && mv tmpfile $file_name
}

#For the moment, this just calls echo. Can be enhanced later
log(){
	echo "LOG [$1]"
}

errorExit() {
    echo; echo -e "\033[31mERROR:\033[0m $1"; echo
    exit 1
}

compareResponse() {
    local MAX_WAIT_TIME=120
    local URL=$1
    local expectedOutput=$2
    local targetNode=$3
    local curlOptions="-s --max-time 60"

    local actualOutput=
    local curl_response=
    # wait for service to be up, check every 10 secs
    # wait for a maximum of 50 seconds, so it doesn't get into an infinite loop
    WAIT_TIME=0
    while [[ "$actualOutput" != *"$expectedOutput"* ]] && [[ $WAIT_TIME -lt $MAX_WAIT_TIME ]] ; do
            echo -n "."
            sleep 10
            WAIT_TIME=$((WAIT_TIME + 10))
            curl_response=$(curl $curlOptions -H "Content-Type: application/json" "$URL" )  || true
            if [[ "$curl_response" != true ]]; then
                actualOutput=$(echo "$curl_response" | jq "$targetNode" ) || true
            fi
    done

    if [[ "$actualOutput" == *"$expectedOutput"* ]]
    then
        return 0
    else
        return 1
    fi
}

#Backward compatible process to delete task if it does not have core url as part of its integration
backwardCompatibleProcess() {
local core_url=$1
if [[ -z "$core_url" || "$core_url" == "" ]]; then
    return
fi

core_url="$core_url/api/tasks/TASKPULLINSIGHT"
# wait until api responds with a success status
compareResponse "$core_url" 'SUCCESS' ".request.status"

if [ $? -ne 0 ]; then
    return
fi

local parameters=$(curl -s "$core_url" | jq '.data.steps.artifactory_insights.parameters')
local param_core_url=
if [[ ! -z "$parameters" && "$parameters" != "" &&  "$parameters" != "null" ]]; then
    param_core_url=$(echo $parameters | jq '.core_url')
    if [[ -z "$param_core_url" || "$param_core_url" == "null" || "$param_core_url" == "" ]]; then
       curl -X DELETE "$core_url" &>/dev/null
    fi
fi
}

log "copying config files to MASTER"
mkdir -p $JFI_HOME_CORE/_MASTER_/data
cp -r $php_app_destination/$vendor_files/config/master/. $JFI_HOME_CORE/_MASTER_/data/


log "Create custom folder if it doesnt exist"

if [ ! -f "$JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch" ]; then
	mkdir -p $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings && \
	cp $JFI_HOME_CORE/_MASTER_/data/contexts/settings/cloudmunch $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch
fi

log "change the owner"
chown -R $core_user:$core_user $JFI_HOME_CORE/platform

log "replace runtime values into the configuration"
replace_value $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch ".|del(.version)"
replace_value $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch ".integrations.scheduler.url=\"$SCHEDULER_URL\""
replace_value $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch ".integrations.executor.url=\"$EXECUTOR_URL\""

[ ! -z "$ELASTIC_SEARCH_URL" ] && replace_value $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch ".integrations.elastic_search.url=\"$ELASTIC_SEARCH_URL\""
[ ! -z "$ELASTIC_SEARCH_USERNAME" ] && replace_value $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch ".integrations.elastic_search.username=\"$ELASTIC_SEARCH_USERNAME\""
[ ! -z "$ELASTIC_SEARCH_PASSWORD" ] && replace_value $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch ".integrations.elastic_search.password=\"$ELASTIC_SEARCH_PASSWORD\""
[ ! -z "$ELASTIC_COMMUNICATION_NODE_URL" ] && replace_value $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch '.integrations.elastic_search.node_ports="[\"'$ELASTIC_COMMUNICATION_NODE_URL'\"]"'
[ ! -z "$ELASTIC_CLUSTER_NAME" ] && replace_value $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch ".integrations.elastic_search.cluster_name=\"$ELASTIC_CLUSTER_NAME\""

replace_value $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch ".integrations.insight_core.url=\"$CORE_URL/api\""
replace_value $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch ".integrations.mission_control.url=\"$JFMC_URL\""
replace_value $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch ".integrations.mongodb.url=\"$MONGO_URL\""
replace_value $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch ".integrations.mongodb.username=\"$MONGODB_USERNAME\""
replace_value $JFI_HOME_CORE/platform/_CUSTOM_/data/contexts/settings/cloudmunch ".integrations.mongodb.password=\"$MONGODB_PASSWORD\""

log "change the memory limit for PHP"
sed -i.bak s@memory_limit\ =\ .*@memory_limit\ =\ 1024M@g /etc/php5/cli/php.ini && \
sed -i.bak s@memory_limit\ =\ .*@memory_limit\ =\ 1024M@g /etc/php5/fpm/php.ini

mkdir -p /tmp && chmod 777 /tmp
chown -R $core_user:$core_user /var/www/html $JFI_HOME_CORE /etc/nginx

[ ! -d "/var/log/php-fpm" ] && mkdir -p /var/log/php-fpm; chown -R $core_user:$core_user /var/log/php-fpm
[ ! -d "/var/log/nginx" ] && mkdir -p /var/log/nginx; chown -R $core_user:$core_user /var/log/nginx

log "Check for temp folder"
if [ ! -d "$JFI_HOME_CORE/domain/_temp_" ]; then
	mkdir -p $JFI_HOME_CORE/domain/_temp_;chown -R $core_user:$core_user $JFI_HOME_CORE/domain
fi

#create template and indexes on Elastic search

log "Start php-fpm in the background and nginx in foreground"
php5-fpm &
nginx -g 'daemon on;'
echo "log start ..." >> /var/log/php-fpm/www-error.log

backwardCompatibleProcess "http://localhost:80"

tail -f /var/log/php-fpm/www-error.log